/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc:fr.bigeon.gclc.command.CommandTestingApplication.java
 * Created on: Jun 12, 2016
 */
package fr.bigeon.gclc;

import java.io.IOException;

import fr.bigeon.gclc.command.ICommand;
import fr.bigeon.gclc.exception.InvalidCommandName;
import fr.bigeon.gclc.manager.PipedConsoleManager;

/**
 * <p>
 * TODO
 *
 * @author Emmanuel Bigeon
 *
 */
@SuppressWarnings("javadoc")
public class CommandTestingApplication implements AutoCloseable {

    private final ConsoleApplication application;
    private final Thread th;
    private final PipedConsoleManager manager;

    /** @throws IOException if the streams cannot be build */
    public CommandTestingApplication() throws IOException {
        manager = new PipedConsoleManager();
        application = new ConsoleApplication(manager, "", "");
        new ConsoleTestApplication().attach(application);
        th = new Thread(new Runnable() {

            @SuppressWarnings("synthetic-access")
            @Override
            public void run() {
                application.start();
            }
        });

        th.start();
    }

    @Override
    public void close() throws IOException {
        application.exit();
        manager.close();
    }

    public void sendCommand(String cmd) throws IOException {
        manager.type(cmd);
    }

    /** @param cmd the command
     * @return if the command was added
     * @throws InvalidCommandName if the command name is invalid
     * @see fr.bigeon.gclc.ConsoleApplication#add(fr.bigeon.gclc.command.ICommand) */
    public final boolean add(ICommand cmd) throws InvalidCommandName {
        return application.add(cmd);
    }

    /** @return the application */
    public ConsoleApplication getApplication() {
        return application;
    }

    /** @throws IOException if the writing failed */
    public void waitAndStop() throws IOException {
        manager.type(ConsoleTestApplication.EXIT);
        try {
            th.join();
        } catch (InterruptedException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
    }

    /** @return the next written line, null if it is the prompt
     * @throws IOException if the reading failed
     * @see fr.bigeon.gclc.manager.PipedConsoleManager#readNextLine() */
    public String readNextLine() throws IOException {
        String ret = manager.readNextLine();
        if (ret.equals(manager.getPrompt())) {
            return null;
        }
        return ret;
    }

}
