/*
 * Copyright Bigeon Emmanuel (2014)
 *
 * emmanuel@bigeon.fr
 *
 * This software is a computer program whose purpose is to
 * provide a generic framework for console applications.
 *
 * This software is governed by the CeCILL license under French law and
 * abiding by the rules of distribution of free software.  You can  use,
 * modify and/or redistribute the software under the terms of the CeCILL
 * license as circulated by CEA, CNRS and INRIA at the following URL
 * "http://www.cecill.info".
 *
 * As a counterpart to the access to the source code and  rights to copy,
 * modify and redistribute granted by the license, users are provided only
 * with a limited warranty  and the software's author,  the holder of the
 * economic rights,  and the successive licensors  have only  limited
 * liability.
 *
 * In this respect, the user's attention is drawn to the risks associated
 * with loading,  using,  modifying and/or developing or reproducing the
 * software by the user in light of its specific status of free software,
 * that may mean  that it is complicated to manipulate,  and  that  also
 * therefore means  that it is reserved for developers  and  experienced
 * professionals having in-depth computer knowledge. Users are therefore
 * encouraged to load and test the software's suitability as regards their
 * requirements in conditions enabling the security of their systems and/or
 * data to be ensured and,  more generally, to use and operate it in the
 * same conditions as regards security.
 *
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL license and that you accept its terms.
 */
/**
 * gclc:fr.bigeon.gclc.tools.PrintUtilsTest.java
 * Created on: Nov 19, 2016
 */
package fr.bigeon.gclc.tools;

import static org.junit.Assert.assertEquals;

import java.util.Arrays;

import org.junit.Test;

/** <p>
 * TODO
 *
 * @author Emmanuel Bigeon */
public class PrintUtilsTest {

    /** Test method for
     * {@link fr.bigeon.gclc.tools.PrintUtils#print(java.lang.String, int, boolean)}. */
    @Test
    public final void testPrint() {
        String text = "Some text";
        String longText = text + " that may be abbreviated";
        assertEquals(text, PrintUtils.print(text, text.length(), true));
        assertEquals(text, PrintUtils.print(text, text.length(), false));
        assertEquals(text + "   ",
                PrintUtils.print(text, text.length() + 3, false));
        assertEquals(text + "   ",
                PrintUtils.print(text, text.length() + 3, true));
        assertEquals(text, PrintUtils.print(longText, text.length(), false));
        assertEquals(text + "...",
                PrintUtils.print(longText, text.length() + 3, true));
    }

    /** Test method for
     * {@link fr.bigeon.gclc.tools.PrintUtils#wrap(java.lang.String, int)}. */
    @Test
    public final void testWrap() {
        String[] line = {"A text separated", "on several lines", "with cuts at",
                "whitespace", "characters"};
        String text = line[0] + " " + line[1] + " " + line[2] + " " + line[3] +
                      " " + line[4];
        assertEquals(Arrays.asList(line),
                PrintUtils.wrap(text, line[0].length() + 1));
        assertEquals(Arrays.asList(line),
                PrintUtils.wrap(text, line[0].length()));

        // test with split word
        line = new String[] {"A text separated", "on several lines", "",
                "with cuts at", "whitespacecharac", "ters"};
        text = line[0] + " " + line[1] + System.lineSeparator() + line[3] +
               " " + line[4] + line[5];
        assertEquals(Arrays.asList(line),
                PrintUtils.wrap(text, line[0].length()));
        assertEquals(Arrays.asList(""),
                PrintUtils.wrap("", line[0].length()));
    }

}
